package com.persagy.bdtp.adm.test;

import cn.hutool.core.util.IdUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.bdtp.adm.dao.*;
import com.persagy.bdtp.adm.entity.*;
import com.persagy.bdtp.adm.service.ISyncModel;
import com.persagy.bdtp.adm.service.ISyncOldAdm;
import com.persagy.bdtp.adm.test.data.InfosCfgs;
import com.persagy.bdtp.adm.test.data.PipeCfgs;
import com.persagy.bdtp.adm.test.data.RelCfgs;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.core.io.Resource;
import org.springframework.core.io.ResourceLoader;
import org.springframework.test.context.junit4.SpringRunner;

import java.util.*;

@RunWith(SpringRunner.class)
@SpringBootTest
public class AdmApplicationTests {

	@Test
	public void contextLoads() {
	}

	@Autowired
	private ISyncOldAdm syncOldAdm;

	@Test
	public void testHandleJobSpace(){
		syncOldAdm.handleJobSpace("Pj0002220002");
	}

	@Test
	public void testSyncOldAdm() {
		syncOldAdm.syncData("BR", "Pj0002220002");
	}

	@Autowired
	private ISyncModel syncModel;

	@Autowired
	private AdmObjectMapper admObjectMapper;

	@Test
	public void testMq(){
		AdmObject obj = admObjectMapper.selectOne(new QueryWrapper<AdmObject>().eq("id", "Eq00022200023a8d5a59a7fb49a89dfa6ae746518bc2"));
		obj.getInfos().put("comp_type", "FFFAAL-声光报警器");
		D2mMessage msg = new D2mMessage();
		msg.setAddItems(Arrays.asList(obj));
		syncModel.sendMessageToModel(msg);
	}

	@Autowired
	private AdmInfosConfigMapper infosConfigMapper;
	@Test
	//@Transactional
	public void testInfosCfg(){
		String type = "ac";

		List<AdmComponent> compList = componentMapper.selectList(null);
		HashMap<String, List<AdmComponent>> compMap = new HashMap<>();
		compList.stream().forEach(comp -> {
			List<AdmComponent> list = compMap.get(comp.getClassCode());
			if(list == null){
				list = new ArrayList<>();
				compMap.put(comp.getClassCode(), list);
			}
			list.add(comp);
		});

		infosConfigMapper.delete(new QueryWrapper<AdmInfosConfig>().eq("flag", type));

		List<AdmInfosConfig> list = new InfosCfgs().read(type);
		for(AdmInfosConfig item : list) {
			item.setFlag(type);

			List<AdmComponent> comps = compMap.get(item.getClassCode());
			if(comps != null)
				setCompsToInfo(item.getInfos(), null, comps);

			infosConfigMapper.insert(item);
		}
	}

	@Autowired
	private AdmRelsConfigMapper relsConfigMapper;
	@Test
	public void testRelsCfg(){
		String type = "ac";
		relsConfigMapper.delete(new QueryWrapper<AdmRelsConfig>().eq("flag", type));
		List<AdmRelsConfig> list = new RelCfgs().read(type);
		for(AdmRelsConfig item : list) {
			item.setFlag(type);
			relsConfigMapper.insert(item);
		}
	}

	@Autowired
	private AdmPipeConfigMapper pipeConfigMapper;
	@Test
	public void testPipeCfg(){
		String type = "ac";
		pipeConfigMapper.delete(new QueryWrapper<AdmPipeConfig>().eq("flag", type));
		List<AdmPipeConfig> list = new PipeCfgs().read(type);
		for(AdmPipeConfig item : list) {
			item.setFlag(type);
			pipeConfigMapper.insert(item);
		}
	}

	@Autowired
	private AdmComponentMapper componentMapper;
	@Test
	public void setCompToInfosCfg(){
		List<AdmComponent> compList = componentMapper.selectList(null);
		HashMap<String, List<AdmComponent>> compMap = new HashMap<>();
		compList.stream().forEach(comp -> {
			List<AdmComponent> list = compMap.get(comp.getClassCode());
			if(list == null){
				list = new ArrayList<>();
				compMap.put(comp.getClassCode(), list);
			}
			list.add(comp);
		});

		List<AdmInfosConfig> cfgList = infosConfigMapper.selectList(null);
		HashMap<String, AdmInfosConfig> cfgMap = new HashMap<>();
		cfgList.stream().forEach(cfg -> cfgMap.put(cfg.getClassCode(), cfg));

		for (Map.Entry<String, List<AdmComponent>> entry : compMap.entrySet()) {
			String classCode = entry.getKey();
			List<AdmComponent> comps = entry.getValue();
			AdmInfosConfig cfg = cfgMap.get(classCode);
			if(cfg != null){
				//不需要更新操作
//				List<Map> infos = (List)cfg.getInfos();
//				Map info = null;
//				for(Map m : infos) {
//					if("comp_type".equals(m.get("code"))){
//						info = m;
//						break;
//					}
//				}
//				setCompsToInfo(infos, info, comps);
//
//				infosConfigMapper.updateById(cfg);
			} else {
				cfg = new AdmInfosConfig();
				cfg.setId(IdUtil.simpleUUID());
				cfg.setObjType("equipment");
				cfg.setClassCode(classCode);
				AdmComponent comp0 = comps.get(0);
				cfg.setSystemCode(comp0.getSystemCode());
				cfg.setMajorCode(comp0.getSystemCode().substring(0, 2));
				List infos = new ArrayList<>();
				cfg.setInfos(infos);
				setCompsToInfo(infos, null, comps);

				infosConfigMapper.insert(cfg);
			}
		}
	}

	private void setCompsToInfo(List infos, Map info, List<AdmComponent> comps){
		if(info == null){
			info = new HashMap();
			info.put("group", "几何信息");
			info.put("code", "comp_type");
			info.put("label", "构件类型");
			info.put("dataType", "string");
			infos.add(info);
		}
		List<Map> ops = new ArrayList<>();
		for(AdmComponent comp : comps) {
			Map<String, String> map = new HashMap<>();
			ops.add(map);
			map.put("val", comp.getCompCode());
			map.put("txt", comp.getCompName());
		}
		info.put("options", ops);
	}

	@Autowired
	ObjectMapper objectMapper;
	@Autowired
	ResourceLoader resourceLoader;
	@Test
	public void testCustomEquip(){
		try{
			Resource resource = resourceLoader.getResource("classpath:equip.json");
			List<TypeDef> types = objectMapper.readValue(resource.getInputStream(), new TypeReference<List<TypeDef>>(){});
			types.stream().forEach(type -> type.setId(IdUtil.simpleUUID()));
			System.out.println(objectMapper.writeValueAsString(types));
		}catch (Exception e) {

		}
	}

	@Test
	public void testAdmObject(){
		try{
			admObjectMapper.selectList(new QueryWrapper<AdmObject>());
		}catch (Exception e) {

		}
	}

}
